/**
* Script name:  TabBarMvc
* 
* Description:  Custom Sencha Touch 2 tab bar that works with a MVC structured application;
*               Full description can be found here:
*               http://www.onlinesolutionsdevelopment.com/blog/web-development/javascript/tab-bar-in-a-sencha-touch-2-mvc-application/
* 
* Author:       CAM
* Author URL:   http://www.onlinesolutionsdevelopment.com/blog/
* 
*/

Ext.define("App.view.TabBarMvc", {
    extend: 'Ext.tab.Bar',
    xtype: 'TabBarMvc',
    config: {
        docked: 'bottom',
        app: null,
        switchAnimation: {
            type: 'slide'
        }
    },
    
    /**
     * Initialize
     */
    initialize: function () {
        this.callParent(arguments);
        
        var thisComponent = this;
        var currentRoute = window.location.hash.substr(1);
        var activeTab = null;
        //
        this.previousTabIndex = 0;
        
        // iterate through all the items and add a tap event handler
        this.getItems().each(function(item, index) {
            item.on('tap', this.tabButtonHandler, this);
            
            if (currentRoute === item.initialConfig.route) {
                activeTab = item;
                this.previousTabIndex = index;
            }
        }, this);
        
        // allow the switch animation to be set as string
        if (Ext.isString( this.getSwitchAnimation() )) {
            this.setSwitchAnimation({
                type: this.getSwitchAnimation()
            });
        }
        
        // set the active tab
        if (activeTab) {
            this.setActiveTab(activeTab);
        }
        
        // detect route changes
        window.addEventListener('hashchange', function () {
            thisComponent.onRouteChange(window.location.hash.substr(1));
        });
    },
    
    /**
     * On tab button tap.
     * @param {Object} tab The tapped button.
     */
    tabButtonHandler: function (tab) {
        var route = tab.initialConfig.route;
        var app = this.getApp();
        var action = null;
        var recognizedRoute = null;
        var anim = {};
        var tabIndex = this.getItems().indexOf(tab);
        
        if (route !== undefined) {
            // parse route
            recognizedRoute = app.getRouter().recognize(route);
            
            if (recognizedRoute) {
                // create the animation
                Ext.apply(anim, this.getSwitchAnimation());
                //
                if (tabIndex != -1 && tabIndex < this.previousTabIndex) {
                    anim.reverse = true;
                }
                //
                recognizedRoute.args.push(anim);
                
                // dispatch route
                action = Ext.create('Ext.app.Action', {
                    url: route,
                    application: app,
                    action: recognizedRoute.action,
                    controller: recognizedRoute.controller,
                    args: recognizedRoute.args
                });
                //
                app.dispatch(action);
                
                this.previousTabIndex = tabIndex;
            }
        }
    },
    
    /**
     * On detected route change.
     * @param {String} route
     */
    onRouteChange: function (route) {
        var items = this.getItems();
        
        // search for the right tab button to select
        items.each(function(item, index) {
            if (route === item.initialConfig.route) {
                this.setActiveTab(item);
                this.previousTabIndex = index;
                return false;
            }
        }, this);
    }
});